/*
	Copyright (c) 2005-2010, Akira Hayashi.
	All rights reserved.

	Redistribution and use in source and binary forms, with or
	without modification, are permitted provided that the following
	conditions are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.

	2. Redistributions in binary form must reproduce the above
	   copyright notice, this list of conditions and the following
	   disclaimer in the documentation and/or other materials provided
	   with the distribution.
	   
	3. Neither the name of the Akira Hayashi, RK Kaihatsu, nor the
	   names of its contributors may be used to endorse or promote
	   products derived from this software without specific prior
	   written permission.
	   
	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
	CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
	MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
	TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
	ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
	THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
	SUCH DAMAGE.
*/

typedef enum RKLineBreakType
{
	kRKLineBreakTypeUnknown,
	kRKLineBreakTypeLF,
	kRKLineBreakTypeCR,
	kRKLineBreakTypeCRLF,
	kRKLineBreakTypeLS
} RKLineBreakType;

typedef enum RKEncodingCode
{
	kRKEncodingCodeUnknown	= 0,
	kRKEncodingCodeSJIS		= 1,
	kRKEncodingCodeJIS		= 2,
	kRKEncodingCodeEUCJP	= 3,
	kRKEncodingCodeUTF8		= 4,
	kRKEncodingCodeUTF16	= 5,
	kRKEncodingCodeUTF32	= 6
} RKEncodingCode;

typedef enum RKByteOrder
{
	kRKByteOrderUnknown,
	kRKByteOrderLE,
	kRKByteOrderBE
} RKByteOrder;

typedef enum RKDeviceCharMapping
{
	kRKDeviceCharMappingMac,
	kRKDeviceCharMappingWindows
} RKDeviceCharMapping;

typedef struct RKEncodingInfo
{
	RKEncodingCode		encodingCode;
	RKByteOrder			byteOrder;
	bool				hasBOM;
	RKDeviceCharMapping	deviceCharMapping;
} RKEncodingInfo;

//
// RKEncodingDetectionWorkingStruct
// The working structure for detecting the text encoding.
//
struct __RKEncodingDetectionWorkingStruct {
	RKEncodingInfo	encodingInfo;	// The encoding information.

	unsigned int	eucFeatures;	// The number of EUC features.
	unsigned int	jisFeatures;	// The number of JIS features.
	unsigned int	utf8Features;	// The number of UTF-8 features.
	BOOL			isNeverUTF8;	// If true, the text encoding is
									// never UTF-8.
	
	const unsigned char*	textBytes;	// The text data.
	const unsigned char*	curBytes;	// The current working bytes.
	unsigned int			textOffset;	// The offset
	unsigned int			textLength;	// The text length.
};
typedef struct __RKEncodingDetectionWorkingStruct RKEncodingDetectionWorkingStruct;

//
// RKTextConverter class declaration
//
@interface RKTextConverter : NSObject {
	NSDictionary*	_halfToFullDullnessMap;
	NSDictionary*	_halfToFullMap;
	NSDictionary*	_halfToFullSemiMap;
}

// Detect the text encoding with BOM.
// If the text encoding was detected returns YES.
- (BOOL)detectEncodingInfoWithBOM:(RKEncodingDetectionWorkingStruct*)workingStruct;

// Check whether any of the UTF-8 features are there at the current
// bytes.
// If it is available returns YES.
- (BOOL)checkUTF8Features:(RKEncodingDetectionWorkingStruct*)workingStruct;

// Check whether any of the UTF-16 or the UTF-32 features are there
// at the current bytes.
// If it is available returns YES.
- (BOOL)checkUTFnFeatures:(RKEncodingDetectionWorkingStruct*)workingStruct;

// If the character is contained in ASCII character returns YES.
- (BOOL)isIncludedInASCII:(unsigned char)c;

// If the character is contained in JIS 0201 Roman set, returns YES.
- (BOOL)isIncludedInJISX0201Roman:(unsigned char)c;

// Returns the line break string.
- (NSString*)lineBreakString:(RKLineBreakType)lineBreakType;

// If the chacter is the half-width katakana, returns YES.
- (BOOL)isHalfwidthKatakana:(unichar)c;

// If the srcChar is a half-width katakana, returns full-width katakana.
// Otherwise, returns srcChar.
- (unichar)convertHalfwidthToFullwidthKatakana:(unichar)srcChar
	nextChar:(unichar)nextChar numOfCharsBeReplaced:(unsigned int*)numOfCharsBeReplaced;

// Create the mapping table for converting half-width to full-width katakana.
- (NSDictionary*)makeHalfToFullMap;

// Create the mapping table for converting half-width to full-width katakana
// with dullness.
- (NSDictionary*)makeHalfToFullDullnessMap;

// Create the mapping table for converting half-width to full-width katakana
// with semi-voiced sound mark.
- (NSDictionary*)makeHalfToFullSemiMap;

//	Create the NSString instance.
- (NSString*)stringWithData:(NSData*)textData encodingInfo:(RKEncodingInfo)encodingInfo;

//	Create the text data.
- (NSData*)dataWithString:(NSString*)str encodingInfo:(RKEncodingInfo)encodingInfo;

// Convert line breaks.
- (void)convertLineBreakOfString:(NSMutableString*)string from:(RKLineBreakType)srcBreakType to:(RKLineBreakType)dstBreakType;

// Get the encoding of the text data.
- (RKEncodingInfo)encodingInfoOfTextData:(NSData*)data; 

@end
